<?php
/* This script is a quick and dirty web interface that allows you to
post a time and have it be displayed in multiple different time zones.
There is no database; everything is specified through GET strings.
There are two pages.  The first is the form that allows you to enter
the time, and the second displays the time in multiple timezones.  It
is a simplified version of what is found on timeanddate.com.

The GET parameters are:

- `h` - the hours (0-23) in the source timezone
- `m` - the minutes (0-59) in the source timezone
- `ampm` - "am" or "pm"
- `tz` - the source timezone (a string, from timezone_identifiers_list())
- `d` - Optionally the day of the month
- `mo` - Optionally the month of the year
- `y` - Optionally the year

TODO:

- There is currently no interface to select the day/month/year

*/

function sorted($a) {
    $t = $a; // Duplicate object
    sort($t);
    return $t;
}

// These cities are usually the reason I would be using this script.
$interesting_cities = array("New Haven" => "America/New_York",
                            "London" => "Europe/London",
                            "Minneapolis" => "America/Chicago",
                            "Indiana" => "America/Indianapolis",
                            "San Francisco" => "America/Los_Angeles",
                            "Athens" => "Europe/Athens",
                            "Moscow" => "Europe/Moscow");

// Display the form to select a date if we don't have all the necessary parameters.
$required_params = array('h', 'm', 'tz', 'ampm');
if (sorted($required_params) != sorted(array_intersect(array_keys($_GET), $required_params))) {
?>
<html>
<head>
<title>Select a date</title>
<link rel="stylesheet" href="style.css" type="text/css">
</head>
<body>
<form action="." method="GET">
<input type="text" size="2" name="h" />:<input type="text" size="2" name="m" />
<select name="ampm" />
<option name="AM" value="AM" selected="selected">AM</option>
<option name="PM" value="PM">PM</option>
<option name="24h" value="24h">24h</option>
</select>
<select name="tz" />
<option name="" selected="selected">Select a timezone...</option>
<optgroup label="Common cities:">
<?php echo implode("\n", array_map(function($k, $t) {echo "<option value=\"$t\">$k</option>";}, array_keys($interesting_cities), array_values($interesting_cities))); ?>
</optgroup>
<optgroup label="All cities:">
<?php echo implode("\n", array_map(function($t) {echo "<option value=\"$t\">$t</option>";}, timezone_identifiers_list())); ?>
</optgroup>
</select>
<input type="submit" value="Submit" />
</form>
</body>
</html>

<?php //End form to select date
exit();
}
// Default value for minutes
$minutes = $_GET['m'] == "" ? 0 : $_GET['m'];

// Check assumptions of script
if (!in_array($_GET['tz'], timezone_identifiers_list())) die("Invalid timezone");
if ($_GET['ampm'] != 'AM' and $_GET['ampm'] != 'PM' and $_GET['ampm'] != '24h') die("Invalid AM/PM");
if ($minutes < 0 or $minutes > 59 or round($minutes) != $minutes) die("Invalid minutes");
if ($_GET['ampm'] == '24h') {
    if ($_GET['h'] < 0 or $_GET['h'] > 23 or round($_GET['h']) != $_GET['h'])
       die("Invalid hours");
} else {
   if ($_GET['h'] < 1 or $_GET['h'] > 12 or round($_GET['h']) != $_GET['h'])
       die("Invalid hours");
}
// Get hours in 24 hour time from AM/PM
if ($_GET['ampm'] == 'PM' and $_GET['h'] != 12) $hours = $_GET['h'] + 12;
else if ($_GET['ampm'] == 'AM' and $_GET['h'] == 12) $hours = 0;
else $hours = $_GET['h'];

// Time conversion is dependent on the date (e.g. DST).  So use today's date if one wasn't supplied.
$date_params = array('d', 'mo', 'y');
if ($date_params == array_intersect(array_keys($_GET), $date_params))
    $date = date_create($_GET['y'] . '/' . $_GET['mo'] . '/' . $_GET['d'] . ' ' . $hours . ':' . $minutes . ':00', timezone_open($_GET['tz']));
else
    $date = date_create('today ' . $hours . ':' . $minutes . ':00', timezone_open($_GET['tz']));
$dateorig = date_create('today ' . $hours . ':' . $minutes . ':00', timezone_open($_GET['tz']));

// Display the table of times
?>
<html>
<head>
<title>Time</title>
<link rel="stylesheet" href="style.css" type="text/css">
<script type="text/javascript" charset="utf-8" src="http://bit.ly/javascript-api.js?version=latest&login=timezonems&apiKey=R_d2a26c47376848c98053619d2ef6d4a3"></script>
<!--Adapted from http://monicams.com/blog/bit-ly-javascript-button/ -->
<script>
shortenURL = function() {
    BitlyClient.shorten(window.location, 'shortenURLcallback');
}
shortenURLcallback = function(data) {
    for (first in data.results) break; // Get first (and only) element in the response object
    document.getElementById("short").innerHTML = data.results[first]['shortUrl'];
}
</script>
</head>
<body>
<button onclick="shortenURL()">Shorten URL</button> <span id="short"></span>
<table>
<?php
// This code is repeated twice close to verbatim, any changes should be propagated.
foreach ($interesting_cities as $key => $z) {
  date_timezone_set($date, timezone_open($z));
  if (date_format($date, 'j') == date_format($dateorig, 'j')) $daystr = "";
  else if (date_offset_get($date) > date_offset_get($dateorig)) $daystr = " (+1 day)";
  else $daystr = " (-1 day)";
  echo "<tr><td>$key</td><td>" . date_format($date, 'g:ia') . "$daystr</td></tr>";
}
echo "<tr><td>&nbsp;</td><td>&nbsp;</td></tr>";
foreach (timezone_identifiers_list() as $z) {
  date_timezone_set($date, timezone_open($z));
  if (date_format($date, 'j') == date_format($dateorig, 'j')) $daystr = "";
  else if (date_offset_get($date) > date_offset_get($dateorig)) $daystr = " (+1 day)";
  else $daystr = " (-1 day)";
  echo "<tr><td>$z</td><td>" . date_format($date, 'g:ia') . "$daystr</td></tr>";
}
?>
</table>
</body>
</html>


<?php // End table of times

?>
